@@ -59,9 +59,7 @@
 #include "misc.h"
 #include "xf86.h"
 #include "xf86Priv.h"
-#ifdef XINPUT
 #include "xf86Xinput.h"
-#endif
 #include "loader.h"
 #include "xf86Optrec.h"
 
@@ -70,8 +68,6 @@
 #include <dirent.h>
 #include <limits.h>
 
-#define TestFree(a) if (a) { xfree (a); a = NULL; }
-
 typedef struct _pattern {
     const char *pattern;
     regex_t rex;
@@ -200,7 +196,6 @@ LoaderSetPath(const char *path)
 /* Standard set of module subdirectories to search, in order of preference */
 static const char *stdSubdirs[] = {
     "",
-    "fonts/",
     "input/",
     "drivers/",
     "multimedia/",
@@ -414,21 +409,21 @@ FindModuleInSubdir(const char *dirpath, const char *module)
         snprintf(tmpBuf, PATH_MAX, "lib%s.so", module);
         if (strcmp(direntry->d_name, tmpBuf) == 0) {
             ret = malloc(strlen(tmpBuf) + strlen(dirpath) + 2);
-            sprintf(ret, "%s/%s", dirpath, tmpBuf);
+            sprintf(ret, "%s%s", dirpath, tmpBuf);
             break;
         }
 
         snprintf(tmpBuf, PATH_MAX, "%s_drv.so", module);
         if (strcmp(direntry->d_name, tmpBuf) == 0) {
             ret = malloc(strlen(tmpBuf) + strlen(dirpath) + 2);
-            sprintf(ret, "%s/%s", dirpath, tmpBuf);
+            sprintf(ret, "%s%s", dirpath, tmpBuf);
             break;
         }
 
         snprintf(tmpBuf, PATH_MAX, "%s.so", module);
         if (strcmp(direntry->d_name, tmpBuf) == 0) {
             ret = malloc(strlen(tmpBuf) + strlen(dirpath) + 2);
-            sprintf(ret, "%s/%s", dirpath, tmpBuf);
+            sprintf(ret, "%s%s", dirpath, tmpBuf);
             break;
         }
     }
@@ -472,7 +467,7 @@ FindModule(const char *module, const char *dirname, const char **subdirlist,
     return name;
 }
 
-_X_EXPORT char **
+char **
 LoaderListDirs(const char **subdirlist, const char **patternlist)
 {
     char buf[PATH_MAX + 1];
@@ -566,7 +561,7 @@ LoaderListDirs(const char **subdirlist, const char **patternlist)
     return listing;
 }
 
-_X_EXPORT void
+void
 LoaderFreeDirList(char **list)
 {
     FreeStringList(list);
@@ -748,13 +743,14 @@ AddSibling(ModuleDescPtr head, ModuleDescPtr new)
     return (new);
 }
 
-_X_EXPORT ModuleDescPtr
-LoadSubModule(ModuleDescPtr parent, const char *module,
+pointer
+LoadSubModule(pointer _parent, const char *module,
 	      const char **subdirlist, const char **patternlist,
 	      pointer options, const XF86ModReqInfo * modreq,
 	      int *errmaj, int *errmin)
 {
     ModuleDescPtr submod;
+    ModuleDescPtr parent = (ModuleDescPtr)_parent;
 
     xf86MsgVerb(X_INFO, 3, "Loading sub module \"%s\"\n", module);
 
@@ -787,12 +783,7 @@ NewModuleDesc(const char *name)
 	mdp->child = NULL;
 	mdp->sib = NULL;
 	mdp->parent = NULL;
-	mdp->demand_next = NULL;
 	mdp->name = xstrdup(name);
-	mdp->filename = NULL;
-	mdp->identifier = NULL;
-	mdp->client_id = 0;
-	mdp->in_use = 0;
 	mdp->handle = -1;
 	mdp->SetupProc = NULL;
 	mdp->TearDownProc = NULL;
@@ -802,7 +793,7 @@ NewModuleDesc(const char *name)
     return (mdp);
 }
 
-_X_EXPORT ModuleDescPtr
+ModuleDescPtr
 DuplicateModule(ModuleDescPtr mod, ModuleDescPtr parent)
 {
     ModuleDescPtr ret;
@@ -817,15 +808,10 @@ DuplicateModule(ModuleDescPtr mod, ModuleDescPtr parent)
     if (LoaderHandleOpen(mod->handle) == -1)
 	return NULL;
 
-    ret->filename = xstrdup(mod->filename);
-    ret->identifier = mod->identifier;
-    ret->client_id = mod->client_id;
-    ret->in_use = mod->in_use;
     ret->handle = mod->handle;
     ret->SetupProc = mod->SetupProc;
     ret->TearDownProc = mod->TearDownProc;
     ret->TearDownData = NULL;
-    ret->path = mod->path;
     ret->child = DuplicateModule(mod->child, ret);
     ret->sib = DuplicateModule(mod->sib, parent);
     ret->parent = parent;
@@ -862,13 +848,6 @@ doLoadModule(const char *module, const char *path, const char **subdirlist,
 
     xf86MsgVerb(X_INFO, 3, "LoadModule: \"%s\"", module);
 
-    for (cim = compiled_in_modules; *cim; cim++)
-	if (!strcmp (module, *cim))
-	{
-	    xf86MsgVerb(X_INFO, 3, "Module \"%s\" already built-in\n", module);
-	    return (ModuleDescPtr) 1;
-	}
-
     patterns = InitPatterns(patternlist);
     name = LoaderGetCanonicalName(module, patterns);
     noncanonical = (name && strcmp(module, name) != 0);
@@ -882,6 +861,15 @@ doLoadModule(const char *module, const char *path, const char **subdirlist,
 	xf86ErrorFVerb(3, "\n");
 	m = (char *)module;
     }
+
+    for (cim = compiled_in_modules; *cim; cim++)
+	if (!strcmp (m, *cim))
+	{
+	    xf86MsgVerb(X_INFO, 3, "Module \"%s\" already built-in\n", m);
+	    ret = (ModuleDescPtr) 1;
+	    goto LoadModule_exit;
+	}
+
     if (!name) {
 	if (errmaj)
 	    *errmaj = LDR_BADUSAGE;
@@ -944,8 +932,6 @@ doLoadModule(const char *module, const char *path, const char **subdirlist,
     if (ret->handle < 0)
 	goto LoadModule_fail;
 
-    ret->filename = xstrdup(found);
-
     /* drop any explicit suffix from the module name */
     p = strchr(name, '.');
     if (p)
@@ -999,7 +985,6 @@ doLoadModule(const char *module, const char *path, const char **subdirlist,
 	    ret->SetupProc = setup;
 	if (teardown)
 	    ret->TearDownProc = teardown;
-	ret->path = path;
 	ret->VersionInfo = vers;
     } else {
 	/* No initdata is OK for external modules */
@@ -1033,19 +1018,10 @@ doLoadModule(const char *module, const char *path, const char **subdirlist,
   LoadModule_exit:
     FreePathList(pathlist);
     FreePatterns(patterns);
-    TestFree(found);
-    TestFree(name);
-    TestFree(p);
+    xfree(found);
+    xfree(name);
+    xfree(p);
 
-    /*
-     * If you need to do something to keep the
-     * instruction cache in sync with the main
-     * memory before jumping to that code, you may
-     * do it here.
-     */
-#ifdef __alpha__
-    istream_mem_barrier();
-#endif
     return ret;
 }
 
@@ -1096,9 +1072,9 @@ LoadModule(const char *module, const char *path, const char **subdirlist,
 }
 
 void
-UnloadModule(ModuleDescPtr mod)
+UnloadModule(pointer mod)
 {
-    UnloadModuleOrDriver(mod);
+    UnloadModuleOrDriver((ModuleDescPtr)mod);
 }
 
 static void
@@ -1120,17 +1096,15 @@ UnloadModuleOrDriver(ModuleDescPtr mod)
 	UnloadModuleOrDriver(mod->child);
     if (mod->sib)
 	UnloadModuleOrDriver(mod->sib);
-    TestFree(mod->name);
-    TestFree(mod->filename);
+    xfree(mod->name);
     xfree(mod);
-#ifdef __alpha__
-    istream_mem_barrier();
-#endif
 }
 
-_X_EXPORT void
-UnloadSubModule(ModuleDescPtr mod)
+void
+UnloadSubModule(pointer _mod)
 {
+    ModuleDescPtr mod = (ModuleDescPtr)_mod;
+
     if (mod == NULL || mod->name == NULL)
 	return;
 
@@ -1145,35 +1119,10 @@ UnloadSubModule(ModuleDescPtr mod)
     if (mod->child)
 	UnloadModuleOrDriver(mod->child);
 
-    TestFree(mod->name);
-    TestFree(mod->filename);
+    xfree(mod->name);
     xfree(mod);
 }
 
-static void
-FreeModuleDesc(ModuleDescPtr head)
-{
-    ModuleDescPtr sibs, prev;
-
-    if (head == (ModuleDescPtr) 1)
-	return;
-    /*
-     * only free it if it's not marked as in use. In use means that it may
-     * be unloaded someday, and UnloadModule will free it
-     */
-    if (head->in_use)
-	return;
-    if (head->child)
-	FreeModuleDesc(head->child);
-    sibs = head;
-    while (sibs) {
-	prev = sibs;
-	sibs = sibs->sib;
-	TestFree(prev->name);
-	xfree(prev);
-    }
-}
-
 static void
 RemoveChild(ModuleDescPtr child)
 {
@@ -1201,7 +1150,7 @@ RemoveChild(ModuleDescPtr child)
     return;
 }
 
-_X_EXPORT void
+void
 LoaderErrorMsg(const char *name, const char *modname, int errmaj, int errmin)
 {
     const char *msg;
